/*
 * ScheduleEditor.java
 * 
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 *
 */

package schedule;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.rmi.*;
import java.io.*;
import java.net.*;
import java.rmi.server.*;
import java.rmi.*;


/**
 * ScheduleEditor
 *     塼δԤȥåץ٥Υ饹
 * @version    1.1 98/05/07
 * @auther     EMOTO Masahiko, Nihon Sun Microsystems K.K.
 */
class ScheduleEditor extends UnicastRemoteObject
implements RemoteScheduleEditor
{
    Frame frame;
    DailyScheduleEditor daily;
    RemoteSchedule schedule = null;
    MonthlyCalendar month;
    String fileName;
    public static String DirName="Schedule";
    final static String URLName = "ScheduleEditor";
    String myName = null;
    
    /**
     *  RMI ФȤ³
     *
     */
    void connectFile(){
	ConnectionDialog diag = new ConnectionDialog(frame);
	diag.setVisible(true);
	RemoteSchedule scd = diag.getSchedule();
	if ( scd == null )
	    return;
	if ( schedule != null ){
	    try {
		schedule.remove(this);
	    }catch (RemoteException ex){
		ErrorDialog errDiag
		    = new ErrorDialog(frame,ErrMsg.getMsg(ErrMsg.ERR_CONNECT));
		errDiag.show();
		ex.printStackTrace();
		return;
	    }
	}
	schedule = scd;
	frame.setTitle(diag.getScheduleName());
	fileName = null;
	daily.update();
	month.show();
    }
    
    /**
     * ץνλ
     *
     */
    void exit() {
	if ( schedule != null){
	    try {
		schedule.remove(this);
	    }catch (RemoteException ex){
		ErrorDialog errDiag
		    = new ErrorDialog(frame,ErrMsg.getMsg(ErrMsg.ERR_REMOTE));
		errDiag.show();
		ex.printStackTrace();
		System.exit(ErrMsg.ERR_REMOTE);
	    }
	}
	try {
	    Naming.unbind(myName);
	} catch (RemoteException ex){
	    ErrorDialog errDiag
		= new ErrorDialog(frame,ErrMsg.getMsg(ErrMsg.ERR_REMOTE));
	    errDiag.show();
	    ex.printStackTrace();
	    System.exit(ErrMsg.ERR_CONNECT);
	} catch (MalformedURLException ex){
	    ErrorDialog errDiag
		= new ErrorDialog(frame,ErrMsg.getMsg(ErrMsg.ERR_CONNECT));
	    errDiag.show();
	    ex.printStackTrace();
	    System.exit(ErrMsg.ERR_CONNECT);
	} catch (NotBoundException ex){
	    ErrorDialog errDiag
		= new ErrorDialog(frame,ErrMsg.getMsg(ErrMsg.ERR_BIND));
	    errDiag.show();
	    ex.printStackTrace();
	    System.exit(ErrMsg.ERR_CONNECT);
	}
	System.exit(0);
    }
    
    /**
     * Connect ν
     *
     */
    class ConnectMenuListener implements ActionListener {
	public ConnectMenuListener () {}
	public void actionPerformed (ActionEvent ev ){
	    connectFile();
	}
    }
    
    /**
     * Exit ν
     *
     */
    class ExitMenuListener implements ActionListener {
	public ExitMenuListener () {}
	public void actionPerformed (ActionEvent ev ){
	    exit();
	}
    }
    
    
    /**
     * դ򤵤줿˸ƤӽФ롣
     *
     */
    void dateSelected(int y,int m, int d) {
	daily.show(y,m,d);
    }
    
    /**
     * 󥹥ȥ饯
     */
    public ScheduleEditor() 
	throws RemoteException {
	    frame = new Frame();
	    daily = new DailyScheduleEditor(this);
	    month = new MonthlyCalendar(this);
	    frame.add("Center",month);
	    ScrollPane pane = new ScrollPane();
	    pane.add(daily);
	    pane.setSize(daily.getSize());
	    frame.add("East",pane);
	    MenuBar bar = new MenuBar();
	    Menu menuFile = new Menu("File");
	    bar.add(menuFile);
	    
	    MenuItem menuItemConnect = new MenuItem("Connect");
	    MenuItem menuItemExit = new MenuItem("Exit");
	    
	    menuItemConnect.addActionListener(new ConnectMenuListener());
	    menuItemExit.addActionListener(new ExitMenuListener());
	    
	    menuFile.add(menuItemConnect);
	    menuFile.addSeparator();
	    menuFile.add(menuItemExit);
	    
	    frame.setMenuBar(bar);
	    
	    frame.setSize(620,300);
	    frame.doLayout();
	    
	}
    
    /**
     * ޥΥФ˼ʬΥ塼䤤碌롣
     */
    void initSchedule() {
	   try {
	    String url = "rmi://localhost/"+ScheduleServer.URLName;
	    String user = System.getProperty("user.name");
	    RemoteScheduleServer server =
		(RemoteScheduleServer )Naming.lookup(url);
	    schedule = server.getSchedule(user);
	    
	}catch(MalformedURLException ex){
	    ErrorDialog diag
		= new ErrorDialog(frame,ErrMsg.getMsg(ErrMsg.ERR_CONNECT));
	    diag.show();
	    ex.printStackTrace();
	}catch(RemoteException ex){
	    ErrorDialog diag
		= new ErrorDialog(frame,ErrMsg.getMsg(ErrMsg.ERR_REMOTE));
	    diag.show();
	    ex.printStackTrace();
	}catch(NotBoundException ex){
	    ErrorDialog diag
		= new ErrorDialog(frame,ErrMsg.getMsg(ErrMsg.ERR_BIND));
	    diag.show();
	    ex.printStackTrace();
	}
	
	daily.update();
	frame.setTitle(System.getProperty("user.name")+"@localhost");
	frame.setVisible(true);
    }
    
    /**
     * 塼빹
     *
     */
    synchronized public void notifyScheduleUpdated(Vector newSchedule) {
	daily.update(newSchedule);
    }
    
    
    Vector getDailyAppointments (int year,int month,int date){
	Vector dailyAppList = null;
	if ( schedule != null ){
	    try {
		dailyAppList
		    = schedule.getDailyAppointments(year,month,date,this);
	    } catch ( java.rmi.RemoteException ex){
		ErrorDialog diag
		    = new ErrorDialog(frame,ErrMsg.getMsg(ErrMsg.ERR_REMOTE));
		diag.show();
		ex.printStackTrace();
	    }
	}
	return dailyAppList;
    }
    
    /**
     *	ݥȥȤι
     *  @param year  ǯ
     *  @param month 
     *  @param date  
     *  @param v     ݥȥ
     *
     */
    void updateDailyAppointments(int year,
				 int month,
				 int date,
				 Vector dailyAppList){
	try {
	    schedule.updateDailyAppointments(year,month,date,dailyAppList);
	}catch ( RemoteException ex){
	    ErrorDialog diag
		= new ErrorDialog(frame,ErrMsg.getMsg(ErrMsg.ERR_CONNECT));
	    diag.show();
	    ex.printStackTrace();
	}
    }
    
    /**
     * Ȥ bind 
     *
     */
    void bindMe() throws RemoteException,MalformedURLException{
	int no = 0;
	boolean bound = false;
	do {
	    bound = true;
	    try {
		myName = new String(URLName)+no;
		Naming.bind(myName,this);
	    } catch ( AlreadyBoundException ex) {
		no++;
		bound = false;
	    }
	} while ( bound == false);
    }
    
    static public void main(String args[]){
	System.setSecurityManager(new RMISecurityManager());
	try {
	    ScheduleEditor mgr = new ScheduleEditor();
	    mgr.bindMe();
	    mgr.initSchedule();
	} catch (RemoteException ex){
	    ex.printStackTrace();
	    System.exit(ErrMsg.ERR_REMOTE);
	} catch (MalformedURLException ex){
	    ex.printStackTrace();
	    System.exit(ErrMsg.ERR_CONNECT);
	}
    }
}
